package sun.rmi.transport;

import sun.rmi.runtime.Log;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;

class WeakRef extends WeakReference<Object>
{
    private int hashValue;
    private Object strongRef;
    
    public WeakRef(Object o) {
        super(o);
        this.strongRef = null;
        this.setHashValue(o);
    }
    
    public WeakRef(Object o, ReferenceQueue<Object> q) {
        super(o, q);
        this.strongRef = null;
        this.setHashValue(o);
    }
    
    public synchronized void pin() {
        if (this.strongRef == null) {
            this.strongRef = this.get();
            if (DGCImpl.dgcLog.isLoggable(Log.VERBOSE)) {
                DGCImpl.dgcLog.log(Log.VERBOSE, "strongRef = " + this.strongRef);
            }
        }
    }
    
    public synchronized void unpin() {
        if (this.strongRef != null) {
            if (DGCImpl.dgcLog.isLoggable(Log.VERBOSE)) {
                DGCImpl.dgcLog.log(Log.VERBOSE, "strongRef = " + this.strongRef);
            }
            this.strongRef = null;
        }
    }
    
    private void setHashValue(Object o) {
        if (o != null) {
            this.hashValue = System.identityHashCode(o);
        }
        else {
            this.hashValue = 0;
        }
    }
    
    @Override
    public int hashCode() {
        return this.hashValue;
    }
    
    @Override
    public boolean equals(Object o) {
        if ((o instanceof WeakRef)) {
            if (o == this) {
                return true;
            }
            Object value = this.get();
            return value != null && value == ((WeakRef)o).get();
        }
        else {
            return false;
        }
    }
}
